@@ -80,7 +80,6 @@ static int deliver_mailbox_file(LOCAL_STATE state, USER_ATTR usr_attr)
     int     mail_copy_status;
     int     deliver_status;
     int     copy_flags;
-    long    end;
     struct stat st;
 
     /*
@@ -132,7 +131,9 @@ static int deliver_mailbox_file(LOCAL_STATE state, USER_ATTR usr_attr)
 	    msg_warn("specify \"%s = no\" to ignore mailbox ownership mismatch",
 		     VAR_STRICT_MBOX_OWNER);
 	} else {
-	    end = vstream_fseek(mp->fp, (off_t) 0, SEEK_END);
+	    if (vstream_fseek(mp->fp, (off_t) 0, SEEK_END) < 0)
+		msg_fatal("%s: seek queue file %s: %m",
+			  myname, VSTREAM_PATH(mp->fp));
 	    mail_copy_status = mail_copy(COPY_ATTR(state.msg_attr), mp->fp,
 					 copy_flags, "\n", why);
 	}
@@ -195,7 +196,7 @@ int     deliver_mailbox(LOCAL_STATE state, USER_ATTR usr_attr, int *statusp)
     mailbox_res = mail_addr_find(virtual_mailbox_maps, state.msg_attr.user,
 				 IGNORE_EXTENSION);
     if (mailbox_res == 0) {
-	if (dict_errno == 0)
+	if (virtual_mailbox_maps->error == 0)
 	    return (NO);
 	msg_warn("table %s: lookup %s: %m", virtual_mailbox_maps->title,
 		 state.msg_attr.user);
